jest.mock("mysql");
import mysql from "mysql";
mysql.createConnection = jest.fn();
mysql.createConnection.mockImplementation(() => {
    return { connect: () => new Promise((resolve, rejects) => resolve(true)) };
});

import saveMessageDatabase from "../test-utils/db/test-saveMessage";
import { generateAttrs, generateUsers } from "../test-utils/test-attrsUtils";
import { jest, describe, it, expect } from "@jest/globals";
import { mocked } from "jest-mock";

import registerMessageInApp, { RoomType } from "../../mails/saveMessage";

const db = new saveMessageDatabase(generateUsers(5));
const ownUser = db.users[0];
const messageId = 1;
const boxId = 1;

jest.mock("../../db/utils/mail", () => {
    return {
        getAddresseId: jest.fn(),
        getUserIdOfMailbox: jest.fn(),
    };
});
jest.mock("../../db/saveMessage-db", () => {
    return {
        createRoom: jest.fn(),
        registerMessageInRoom: jest.fn(),
        isRoomGroup: jest.fn(),
        findRoomsFromMessage: jest.fn(),
        hasSameMembersAsParent: jest.fn(),
        registerThread: jest.fn(),
        registerMember: jest.fn(),
        getAllMembers: jest.fn(),
        getThreadInfo: jest.fn(),
        incrementNotSeenRoom: jest.fn(),
    };
});
import { getAddresseId, getUserIdOfMailbox } from "../../db/utils/mail";
import {
    createRoom,
    registerMessageInRoom,
    isRoomGroup,
    findRoomsFromMessage,
    hasSameMembersAsParent,
    registerThread,
    registerMember,
    getAllMembers,
    getThreadInfo,
    incrementNotSeenRoom,
} from "../../db/saveMessage-db";
// todo esbuild
// new message from us
//     to multiple people -> room
//         if response has same member => group
//         if response is dm => channel
//     to one person => dm

// new message from other
//     to only me -> room
//         if no reply to multiple message => channel
//         else => dm
//     to multiple people -> room
//         // make it better
//         if multiple members reply -> group
//         if only me reply -> channel

beforeAll(async () => {
    db.clear();
    mocked(getAddresseId).mockImplementation(db.getAddresseId);
    mocked(getUserIdOfMailbox).mockImplementation(db.getUserIdOfMailbox);

    mocked(createRoom).mockImplementation(db.createRoom);
    mocked(registerMessageInRoom).mockImplementation(db.registerMessageInRoom);
    mocked(isRoomGroup).mockImplementation(db.isRoomGroup);
    mocked(findRoomsFromMessage).mockImplementation(db.findRoomsFromMessage);
    mocked(hasSameMembersAsParent).mockImplementation(db.hasSameMembersAsParent);
    mocked(registerThread).mockImplementation(db.registerThread);
    mocked(registerMember).mockImplementation(db.registerMember);
    mocked(getAllMembers).mockImplementation(db.getAllMembers);
    mocked(getThreadInfo).mockImplementation(db.getThreadInfo);
    mocked(incrementNotSeenRoom).mockImplementation(db.incrementNotSeenRoom);
});

describe("saveMessage", () => {
    describe("functions", () => {
        it("isFromUs", async () => {
            const attrs = generateAttrs({ from: [ownUser.user], to: [db.users[1].user] });
            const register = new registerMessageInApp(messageId, attrs, boxId);
            await register.init();
            const res = await register.isFromUs();

            expect(res).toBe(true);

            const attrs2 = generateAttrs({ from: [db.users[2].user], to: [db.users[1].user] });
            const register2 = new registerMessageInApp(messageId, attrs2, boxId);
            await register2.init();
            const res2 = await register2.isFromUs();

            expect(res2).toBe(false);
        });
    });
    describe("implementation", () => {
        describe("new first message from us", () => {
            it("should create a DM when there is a new first message from us to one recipient", async () => {
                const attrs = generateAttrs({ from: [ownUser.user], to: [db.users[1].user] });

                const register = new registerMessageInApp(messageId, attrs, boxId);

                const createOrRegisterOnExistence = jest
                    .spyOn(register, "createOrRegisterOnExistence")
                    .mockImplementation(
                        (owner: number, roomType: RoomType) => new Promise((resolve, reject) => resolve()),
                    );

                await register.save();

                expect(createOrRegisterOnExistence).toHaveBeenCalledWith(db.users[1].id, RoomType.DM);
            });
            it("should create a ROOM when there is a new first message from us to multiple recipients", async () => {
                const attrs = generateAttrs({ from: [ownUser.user], to: [db.users[1].user, db.users[2].user] });

                const register = new registerMessageInApp(messageId, attrs, boxId);

                const initiateRoom = jest
                    .spyOn(register, "initiateRoom")
                    .mockImplementation((owner: number, roomType: RoomType) => Promise.resolve(1));

                await register.save();

                expect(initiateRoom).toHaveBeenCalledWith(ownUser.id, RoomType.ROOM);
            });
            // it("response to new first message to multiple recipients with same members should change room type to GROUP", () => {

            // });
            // it("response to new first message to multiple recipients with different members should change room type to CHANNEL", () => {

            // });
        });
        describe("new first message from other", () => {
            it("should create a ROOM when there is a new first message from other to me only", async () => {
                const attrs = generateAttrs({ from: [db.users[1].user], to: [ownUser.user] });

                const register = new registerMessageInApp(messageId, attrs, boxId);

                const createOrRegisterOnExistence = jest
                    .spyOn(register, "createOrRegisterOnExistence")
                    .mockImplementation((owner: number, roomType: RoomType) => {
                        return new Promise((resolve, reject) => resolve());
                    });

                await register.save();

                expect(createOrRegisterOnExistence).toHaveBeenCalledWith(db.users[1].id, RoomType.ROOM);
            });
        });
        describe("replies", () => {
            it("", () => {});
        });
    });
    describe("unseen behavior", () => {
        it("should add unseen in room when a message creates a room", () => {
            
        });

        it("should add unseen in room when a message joins a room", () => {
            
        });

        it("should add unseen in root room and thread when new message creates a thread", () => {
            
        });

        it("should add unseen in root room and thread when new message joins in thread", () => {
            
        });
    });
});
